@@ -240,6 +240,20 @@ def write_info_json(output_path, parsed_question):
         parsed_question (dict]): [description]
     """
 
+    # Deal with optional tags in info.json
+    optional = ""
+
+    if parsed_question['header'].get('gradingMethod'):
+        optional += """ "gradingMethod": parsed_question['header']['gradingMethod'],\n"""
+    elif parsed_question['header'].get('partialCredit'):
+        optional += """ "partialCredi": parsed_question['header']['partialCredit'],\n"""
+    elif parsed_question['header'].get('externalGradingOptions'):
+        optional += """ "externalGradingOptions": parsed_question['header']['externalGradingOptions'],\n"""
+    elif parsed_question['header'].get('dependencies'):
+        optional += """ "dependencies": parsed_question['header']['dependencies'],\n"""
+    elif parsed_question['header'].get('singleVariant'):
+        optional += """ "singleVariant": parsed_question['header']['singleVariant'],\n"""
+
     pathlib.Path(output_path / 'info.json').write_text("""{
             "uuid": \"""" + str(uuid.uuid3(uuid.NAMESPACE_DNS, str(output_path))) + """\",
             "title": \"""" + parsed_question['header']['title'] + """",
@@ -375,6 +389,24 @@ def process_symbolic_input(part_name,parsed_question, data_dict):
 
     return replace_tags(html).replace('\\\\','\\')
 
+def process_longtext(part_name,parsed_question, data_dict):
+    """Processes markdown format of long-text questions and returns PL HTML 
+    Args:
+        part_name (string): Name of the question part being processed (e.g., part1, part2, etc...)
+        parsed_question (dict): Dictionary of the MD-parsed question (output of `read_md_problem`)
+        data_dict (dict): Dictionary of the `data` dict created after running server.py using `exec()`
+
+    Returns:
+        html: A string of HTML that is part of the final PL question.html file.
+    """
+    pl_customizations = " ".join([f'{k} = "{v}"' for k,v in parsed_question['header'][part_name]['pl-customizations'].items()]) # PL-customizations
+
+    html = f"""<pl-question-panel>\n<markdown>{parsed_question['body_parts_split'][part_name]['content']}</markdown>\n</pl-question-panel>\n\n"""
+
+    html += f"""<pl-rich-text-editor { pl_customizations } > </pl-rich-text-editor>"""
+
+    return replace_tags(html)
+
 def replace_tags(string):
     """Takes in a string with tags: |@ and @| and returns {{ and }} respectively. This is because Python strings can't have double curly braces.
 
@@ -435,11 +467,11 @@ def process_attribution(source):
 
         elif 'standard' in source:
             attribution_text = "Problem is licensed under the [CC-BY-NC-SA 4.0 license](https://creativecommons.org/licenses/by-nc-sa/4.0/).<br> ![The Creative Commons 4.0 license requiring attribution-BY, non-commercial-NC, and share-alike-SA license.](https://raw.githubusercontent.com/firasm/bits/master/by-nc-sa.png)"
-    
+
         return attribution_text
     
     except TypeError:
-        print("You probably need to update the template, the 'attribution' key seems to be missing.")
+        print(f"`source` value of {source} not recognized. You probably need to update the template, the 'source' key seems to be missing or not recognized.")
         
 def process_question_md(source_filepath, output_path = None, instructor = False):
     
@@ -574,6 +606,10 @@ def process_question_pl(source_filepath, output_path = None):
     ## Single part questions
     if parsed_q['num_parts'] == 1:
         q_type = parsed_q['header']['part1']['type']
+
+        ## Add code to make sure correct answer is not shown by default (START of hide-in-panel)
+        question_html += '<pl-hide-in-panel answer="true">\n'
+        
         if 'multiple-choice' in q_type:
             question_html += process_multiple_choice('part1',parsed_q,data2)
         elif 'number-input' in q_type:
@@ -584,15 +620,23 @@ def process_question_pl(source_filepath, output_path = None):
             question_html += process_symbolic_input('part1',parsed_q,data2)
         elif 'dropdown' in q_type:
             question_html += process_dropdown('part1',parsed_q,data2)
+        elif 'longtext' in q_type:
+            question_html += process_longtext('part1',parsed_q,data2)
         else:
             raise NotImplementedError(f"This question type ({q_type}) is not yet implemented.")
 
+        ## Add code to make sure correct answer is not shown by default (END of hide-in-panel)
+        question_html += '</pl-hide-in-panel>\n'
+
     ##### Multi part
     else:
         for pnum in range(1, parsed_q['num_parts'] + 1):
             part = 'part'+f'{pnum}'
             q_type = parsed_q['header'][part]['type']
 
+            ## Add code to make sure correct answer is not shown by default (START of hide-in-panel)
+            question_html += '<pl-hide-in-panel answer="true">\n'
+
             question_html += f"""<div class="card my-2">
 <div class="card-header">{parsed_q['body_parts_split'][part]['title']}</div>\n
 <div class="card-body">\n\n"""
@@ -607,11 +651,16 @@ def process_question_pl(source_filepath, output_path = None):
                 question_html += process_symbolic_input(part,parsed_q,data2)
             elif 'dropdown' in q_type:
                 question_html += process_dropdown(part,parsed_q,data2)
+            elif 'longtext' in q_type:
+                question_html += process_longtext(part,parsed_q,data2)
             else:
                 raise NotImplementedError(f"This question type ({q_type}) is not yet implemented.")
 
             question_html += "</div>\n</div>\n\n\n"
 
+            ## Add code to make sure correct answer is not shown by default (END of hide-in-panel)
+            question_html += '</pl-hide-in-panel>\n'
+            
     # Add pl-submission-panel and pl-answer-panel (if they exist)
     subm_panel = parsed_q['body_parts_split'].get('pl-submission-panel', None)
     q_panel = parsed_q['body_parts_split'].get('pl-answer-panel', None)
@@ -653,11 +702,21 @@ def pl_image_path(html):
     """Adds `{{options.client_files_question_url}}` directory before the path automatically
     """
 
+    # TODO: Figure out the regex to make this into a single expression, maybe with |
     # If image files are included as markdown format, add {{options.client_files_question_url}}
     res = re.subn(r"\(((?!http).*\.png)\)",'({{options.client_files_question_url}}/\\1)',html)
+    res = re.subn(r"\(((?!http).*\.gif)\)",'({{options.client_files_question_url}}/\\1)',html)
+    res = re.subn(r"\(((?!http).*\.jpeg)\)",'({{options.client_files_question_url}}/\\1)',html)
+    res = re.subn(r"\(((?!http).*\.jpg)\)",'({{options.client_files_question_url}}/\\1)',html)
 
     # If image files are included as html format, add {{options.client_files_question_url}}
     res = re.subn(r"src[\s,=]*\"(?!http)(.*\.png)",
               "src=\"{{options.client_files_question_url}}/\\1",res[0]) # works
+    res = re.subn(r"src[\s,=]*\"(?!http)(.*\.gif)",
+              "src=\"{{options.client_files_question_url}}/\\1",res[0]) # works
+    res = re.subn(r"src[\s,=]*\"(?!http)(.*\.jpeg)",
+              "src=\"{{options.client_files_question_url}}/\\1",res[0]) # works
+    res = re.subn(r"src[\s,=]*\"(?!http)(.*\.jpg)",
+              "src=\"{{options.client_files_question_url}}/\\1",res[0]) # works
 
     return res[0]
\ No newline at end of file
